<?php

require_once($_SERVER['DOCUMENT_ROOT'] . "/conf/autoloader.php");

$_message = NULL;

try {

    header('Content-Type: text/event-stream');
    header('Cache-Control: no-cache');

    ob_end_clean();

    $session = new sessionmanager();
    $token = $session->getToken();
    $userID = $session->verifySession($token);
    $mqtt = new Mosquitto\Client(1);

    register_shutdown_function(function () use ($mqtt, $session) {
        try {
            $mqtt->disconnect();
        } catch (Exception $ex) {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);
            return false;
        }
    });

    if ($userID) {

        $notifications = array();
        $mqtt->connect('localhost', 1883, 60);
        $mqtt->subscribe(PROP_SERVICE_PROP_CLIENT_TOPIC, 1);

        $mqtt->onMessage(function ($_message) use (&$notifications, &$_timeout) {
            $_timeout = microtime(true);
            $id = round($_timeout * 1000);
            $msg = createMsg($id, $_message);
            isMsgNotEmpty($msg) ? $notifications[] = $msg : null;
        });

        while (true) {
            $mqtt->loop();
            $token = $session->getToken();

            if ($session->verifySession($token)) {
                if (count($notifications) > 0) {
                    sendMsgNottify($notifications);
                    $notifications = array();
                } else {
                    echo ":" . PHP_EOL . PHP_EOL;
                }
                flush();
            } else {
                die();
            }
            usleep(300000);
        }
    }
} catch (Exception $ex) {
    $exc = new errorexceptionhandler();
    $exc->getException($ex);
    return false;
}

function sendMsg($msg)
{
    echo PHP_EOL;
    echo "event: paramUpdate\n";
    echo "data: $msg \n";
    echo PHP_EOL;
}

function sendMsgNottify($notiffy)
{
    foreach ($notiffy as $_msg) {
        sendMsg(json_encode($_msg));
    }
}

function createMsg($id, $_message)
{
    $payload = json_decode($_message->payload, false);
    $_msg = new stdClass();
    $_msg->topic = $_message->topic;

    if ($payload->command === PARAM_CHANGE_COMMAND) {

        $_msg->id = $id;
        $_msg->topic = $_message->topic;

        if (sizeof($payload->parameters->list) == 1) {
            if (!empty($payload->parameters->list[0]->keyWebui)) {
                $_msg->key = $payload->parameters->list[0]->keyWebui;
            } else {
                $tmpMsgArray = explode('.', $payload->parameters->list[0]->key);
                ($tmpMsgArray[0] == 'State') ? $_msg->key = $payload->parameters->list[0]->key : $_msg->key = NULL;
            }
            $_msg->value = (isset($payload->parameters->list[0]->value)) ? $payload->parameters->list[0]->value : NULL;
        } else {
            $_msg->payload = prepareProvisioningParameters($payload->parameters->list);
        }
    }

    return $_msg;
}

function prepareProvisioningParameters($messageArray)
{
    $parameters = [];

    foreach ($messageArray as $parameter) {
        if (!empty($parameter->keyWebui)) {
            $parameters[] = (object)[
                'key' => $parameter->keyWebui,
                'value' => $parameter->value
            ];
        }
    }
    return $parameters;
}

function isMsgNotEmpty($msg)
{
    return (!empty($msg->payload) || !empty($msg->key));
}

?>